@@ -1,4 +1,5 @@
 import logging
+import os
 import smtplib
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
@@ -14,17 +15,28 @@ def send_markdown_email(auth_url, markdown_content, subject, to_address):
     html_content = markdown2.markdown(markdown_content)
 
     msg = MIMEMultipart()
-    msg["From"] = parsed_url.username
+    msg["From"] = os.environ.get("EMAIL_FROM", parsed_url.username)
     msg["To"] = to_address
     msg["Subject"] = subject
 
-    # TODO any sane styling we can setup?
+    logger.info(
+        "creating email for %s, from %s, content length %i",
+        to_address,
+        parsed_url.username,
+        len(markdown_content),
+    )
+
+    # this caused the html version to be ignored...
+    # msg.attach(MIMEText(markdown_content, "plain"))
+
+    # TODO any sane HTML styling we can setup?
     msg.attach(MIMEText(html_content, "html"))
 
     with smtplib.SMTP_SSL(parsed_url.hostname, parsed_url.port) as server:
         logger.info("Sending email to %s", to_address)
 
-        server.login(parsed_url.username, parsed_url.password)
+        login_result = server.login(parsed_url.username, parsed_url.password)
+        logger.info("Login result: %s", login_result)
         result = server.send_message(msg)
+        logger.info("Send result: %s", result)
         server.quit()
-        breakpoint()
