@@ -1,7 +1,12 @@
+import logging
+
 import backoff
 import requests
 import todoist_api_python.http_requests
 
+# backoff does not log by default
+logging.getLogger("backoff").addHandler(logging.StreamHandler())
+
 
 # https://github.com/Doist/todoist-api-python/issues/38
 # backoff 5xx errors
@@ -16,8 +21,9 @@ def patch_todoist_api():
             original_function,
         )
 
+        # TODO pretty sure authorization errors are retried :/
         patched_function = backoff.on_exception(
-            backoff.expo, requests.exceptions.HTTPError
+            backoff.expo, requests.exceptions.HTTPError, max_tries=5
         )(original_function)
 
         setattr(
